//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#ifndef DEVDEFS_H
#define DEVDEFS_H

#include "../types.h"
#include "../config.h"

#define DEVFLAGS_ACTIVE 				0x00000001
#define DEVFLAGS_USES_IO				0x00000002
#define DEVFLAGS_USES_IRQ				0x00000004
#define DEVFLAGS_USES_DMA				0x00000008
#define DEVFLAGS_USES_MEMORY			0x00000010

typedef enum
{
	devDisplayAdapter = 0,
	devUnknown = 255
} devDeviceTypes;

typedef enum
{
	devISA = 0,
	devPCI = 1,
	devAGP = 2,
	devBusUnknown = 255
} devBusTypes;

typedef enum
{
	devByte = 1,
	devHalfword = 2,
	devWord = 4
} devDataSizeTypes;

typedef struct devDeviceDefinition
{
	devDeviceTypes devType;
	devBusTypes devBus;
	char    *devName;
	char    *devModelID;
	ulong    devFlags;

	uint     devPortCount, devIRQCount, devDMACount, devMemoryCount;
	ushort  *devBeginPortList,   *devEndPortList;
	uchar   *devBeginIRQList,    *devEndIRQList;
	uchar   *devBeginDMAList,    *devEndDMAList;
	ulong   *devBeginMemoryList, *devEndMemoryList;

	bool   (*devInitFunc)(devDeviceDefinition *devBlk);
	void   (*devShutdownFunc)(devDeviceDefinition *devBlk);
	ulong  (*devReadIOFunc)(ushort port, devDataSizeTypes dataSize);
	void   (*devWriteIOFunc)(ushort port, ulong val, devDataSizeTypes dataSize);
} devDeviceDefinition;

#endif
