//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "../config.h"
#include "../types.h"
#include "devdefs.h"
#include "oak.h"
#include "../devices.h"
#include "../video.h"
#include <stdlib.h>


static char *deviceName = "Oak Technology VGA Adapter";
static char *modelID    = "OTVGA-1";

void oakRegisterDevice(devDeviceDefinition *devBlk)
{
	// Fill out device definition...
	devBlk->devType         = devDisplayAdapter;
	devBlk->devBus          = devISA;
	devBlk->devName         = deviceName;
	devBlk->devModelID      = modelID;
	devBlk->devFlags        = DEVFLAGS_USES_IO | DEVFLAGS_USES_MEMORY;
	devBlk->devInitFunc     = oakInit;
	devBlk->devShutdownFunc = oakShutdown;
	devBlk->devReadIOFunc   = oakReadIO;
	devBlk->devWriteIOFunc  = oakWriteIO;

	// Allocate port/memory lists...
	devBlk->devBeginPortList   = (ushort *)malloc(2 * sizeof(ushort));
	devBlk->devEndPortList     = (ushort *)malloc(2 * sizeof(ushort));
	devBlk->devBeginMemoryList = (ulong  *)malloc(1 * sizeof(ulong ));
	devBlk->devEndMemoryList   = (ulong  *)malloc(1 * sizeof(ulong ));

	// Fill out lists...
	devBlk->devPortCount = 2;
	devBlk->devBeginPortList[0] = 0x3C0;  devBlk->devEndPortList[0] = 0x3CF;
	devBlk->devBeginPortList[1] = 0x3D5;  devBlk->devEndPortList[1] = 0x3D5;

	devBlk->devMemoryCount = 1;
	devBlk->devBeginMemoryList[0] = 0x000B8000;  devBlk->devEndMemoryList[0] = 0x000BBFFF;

	devBlk->devIRQCount = 0;
	devBlk->devDMACount = 0;
}


bool oakInit(devDeviceDefinition *devBlk)
{
	if (devRequestPorts(devBlk) == FALSE)
	{
		// didn't get all i/o ports... some must be in use!
		return FALSE;
	}

	if (devRequestMemory(devBlk) == FALSE)
	{
		// didn't get all memory addresses... some must be in use!
		return FALSE;
	}

	// don't call videoInit!  it will already be done before here...
	return TRUE;
}


void oakShutdown(devDeviceDefinition *devBlk)
{
}


ulong oakReadIO(ushort port, devDataSizeTypes dataSize)
{
	videoSetMode(320, 200);
	return 0;
}


void oakWriteIO(ushort port, ulong val, devDataSizeTypes dataSize)
{
	videoBlit(NULL);
}
