//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "config.h"
#include "types.h"
#include "r59basic.h"


ulong r5900ADDI(uchar rt, uchar rs, ushort immed)
{
	return (0x08 << 26) | (rs << 21) | (rt << 16) | immed;
}

ulong r5900ADDIU(uchar rt, uchar rs, ushort immed)
{
	return (0x09 << 26) | (rs << 21) | (rt << 16) | immed;
}

ulong r5900ADD(uchar rd, uchar rs, uchar rt)
{
	return (rs << 21) | (rt << 16) | (rd << 11) | 0x20;
}

ulong r5900ADDU(uchar rd, uchar rs, uchar rt)
{
	return (rs << 21) | (rt << 16) | (rd << 11) | 0x21;
}

ulong r5900OR(uchar rd, uchar rs, uchar rt)
{
	return (rs << 21) | (rt << 16) | (rd << 11) | 0x25;
}

ulong r5900ORI(uchar rt, uchar rs, ushort immed)
{
	return (0x0D << 26) | (rs << 21) | (rt << 16) | immed;
}

ulong r5900AND(uchar rd, uchar rs, uchar rt)
{
	return (rs << 21) | (rt << 16) | (rd << 11) | 0x24;
}

ulong r5900ANDI(uchar rt, uchar rs, ushort immed)
{
	return (0x0C << 26) | (rs << 21) | (rt << 16) | immed;
}

ulong r5900XOR(uchar rd, uchar rs, uchar rt)
{
	return (rs << 21) | (rt << 16) | (rd << 11) | 0x26;
}

ulong r5900XORI(uchar rt, uchar rs, ushort immed)
{
	return (0x0E << 26) | (rs << 21) | (rt << 16) | immed;
}

ulong r5900LUI(uchar rt, ushort immed)
{
	return (0x0F << 26) | (rt << 16) | immed;
}

ulong r5900SLL(uchar rd, uchar rt, uchar sa)
{
	return (rt << 16) | (rd << 11) | (sa << 6);
}

ulong r5900SRL(uchar rd, uchar rt, uchar sa)
{
    return (rt << 16) | (rd << 11) | (sa << 6) | 0x3;
}

ulong r5900SB(uchar rt, uchar base, ushort offset)
{
	return (0x28 << 26) | (base << 21) | (rt << 16) | offset;
}

ulong r5900JR(uchar rs)
{
	return (rs << 21) | 0x8;
}
