//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#ifndef X86_H
#define X86_H

#include "types.h"

// helpful defines for decoding...
#define REG_IS_LOW(r)					( (r) <= xBL )
#define REG_IS_HIGH(r)					( (r) >= xAH )

#define MODRM_IS_REGISTER(x)			(  ((x) & 0x18) == 0x18 )						// bitmask: 00011000  (in modrm: 11 xxx)
#define MODRM_GET_REG_BYTE(x)			((x86RegByte) ((x) & 0x07))						// bitmask: 00000111  (in modrm: 00 xxx)
#define MODRM_HAS_DISPL8(x)				(  ((x) & 0x18) == 0x08 )						// bitmask: 00001000  (in modrm: 01 xxx)
#define MODRM_HAS_DISPL16(x)			( (((x) & 0x18) == 0x10) | ((x) == 0x06) )		// bitmask: 00010000  (in modrm: 10 xxx)
#define MODRM_HAS_DISPL32(x)			( (((x) & 0x18) == 0x10) | ((x) == 0x05) )		// bitmask: 00010000  (in modrm: 10 xxx)
#define MODRM_HAS_SIB(x)				(  ((x) & 0x07) == 0x04 )						// bitmask: 00000100  (in modrm: xx 100)

#define DECODE_REG_BYTE_FIELD(x)		((x86RegByte)       (( (x) >> 3 ) & 0x07 ))
#define DECODE_MODRM_FIELD(x)			((x86MODRMByte16) (((( (x) >> 6 ) & 0x03 ) << 3) | ((x) & 0x07)) )

#define DECODE_SIB_MODBASE(dEA, x)		((x86MODRMByte32SIB) (((dEA) & 0x18) | ((x) & 0x07)) )
#define DECODE_SIB_SS(x)				(   (x)         >> 5 )
#define DECODE_SIB_INDEX(x)				(  ((x) & 0x38) >> 3 )

#define SIB_HAS_DISPL8(x)				(  ((x) & 0x18) == 0x08 )						// bitmask: 00001000  (in modbase: 01 xxx)
#define SIB_HAS_DISPL32(x)				(  ((x) & 0x18) == 0x10 )						// bitmask: 00010000  (in modbase: 10 xxx)


// enums for register usage when decoding...
typedef enum
{
	xAL = 0,
	xCL,
	xDL,
	xBL,
	xAH,
	xCH,
	xDH,
	xBH
} x86RegByte;

typedef enum
{
	xAX = 0,
	xCX,
	xDX,
	xBX,
	xSP,
	xBP,
	xSI,
	xDI
} x86RegWord16;

typedef enum
{
	xEAX = 0,
	xECX,
	xEDX,
	xEBX,
	xESP,
	xEBP,
	xESI,
	xEDI
} x86RegWord32;


// enums for handling effective address decoding...
typedef enum
{
	ea16DS_BXpSI = 0,
	ea16DS_BXpDI,
	ea16SS_BPpSI,
	ea16SS_BPpDI,
	ea16DS_SI,
	ea16DS_DI,
	ea16DS_d16,
	ea16DS_BX,

	ea16DS_BXpSIpd8,
	ea16DS_BXpDIpd8,
	ea16SS_BPpSIpd8,
	ea16SS_BPpDIpd8,
	ea16DS_SIpd8,
	ea16DS_DIpd8,
	ea16SS_BPpd8,
	ea16DS_BXpd8,

	ea16DS_BXpSIpd16,
	ea16DS_BXpDIpd16,
	ea16SS_BPpSIpd16,
	ea16SS_BPpDIpd16,
	ea16DS_SIpd16,
	ea16DS_DIpd16,
	ea16SS_BPpd16,
	ea16DS_BXpd16,

	ea16_rAL,
	ea16_rCL,
	ea16_rDL,
	ea16_rBL,
	ea16_rAH,
	ea16_rCH,
	ea16_rDH,
	ea16_rBH
} x86MODRMByte16;

typedef enum
{
	ea32DS_EAX = 0,
	ea32DS_ECX,
	ea32DS_EDX,
	ea32DS_EBX,
	ea32_hasSIB1,
	ea32DS_d32,
	ea32DS_ESI,
	ea32DS_EDI,

	ea32DS_EAXpd8,
	ea32DS_ECXpd8,
	ea32DS_EDXpd8,
	ea32DS_EBXpd8,
	ea32_hasSIB2,
	ea32SS_EBPpd8,
	ea32DS_ESIpd8,
	ea32DS_EDIpd8,

	ea32DS_EAXpd32,
	ea32DS_ECXpd32,
	ea32DS_EDXpd32,
	ea32DS_EBXpd32,
	ea32_hasSIB3,
	ea32SS_EBPpd32,
	ea32DS_ESIpd32,
	ea32DS_EDIpd32,

	ea32_rAL,
	ea32_rCL,
	ea32_rDL,
	ea32_rBL,
	ea32_rAH,
	ea32_rCH,
	ea32_rDH,
	ea32_rBH
} x86MODRMByte32;

typedef enum
{
	ea32sibDS_EAX = 0,
	ea32sibDS_ECX,
	ea32sibDS_EDX,
	ea32sibDS_EBX,
	ea32sibSS_ESP,
	ea32sibDS_d32,
	ea32sibDS_ESI,
	ea32sibDS_EDI,

	ea32sibDS_EAXpd8,
	ea32sibDS_ECXpd8,
	ea32sibDS_EDXpd8,
	ea32sibDS_EBXpd8,
	ea32sibSS_ESPpd8,
	ea32sibSS_EBPpd8,
	ea32sibDS_ESIpd8,
	ea32sibDS_EDIpd8,

	ea32sibDS_EAXpd32,
	ea32sibDS_ECXpd32,
	ea32sibDS_EDXpd32,
	ea32sibDS_EBXpd32,
	ea32sibSS_ESPpd32,
	ea32sibSS_EBPpd32,
	ea32sibDS_ESIpd32,
	ea32sibDS_EDIpd32
} x86MODRMByte32SIB;


// enums for segment/selector overrides...
typedef enum
{
	xCS = 0,
	xDS,
	xES,
	xSS,
	xFS,
	xGS
} x86Seg;


// useful lookup tables...
extern uchar x86SegToR5900Reg[6];
extern uchar x86RegByteToR5900Reg[8];
extern uchar x86RegWordToR5900Reg[8];


// global variables...
extern bool   gIn32BitMode;
extern bool   gIsOpSize32, gIsAddrSize32, gIsSegOverride;
extern uchar  gOpSizeCounter, gAddrSizeCounter, gSegOvrCounter;
extern x86Seg gOverrideSeg;


// the big table...
extern int (*x86OpcodeTable[256])(uchar *in, ulong *out, bool *isBranch, ulong *skip);

#endif
