//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "config.h"
#include "types.h"
#include "x86.h"
#include "x86bit.h"
#include "r59basic.h"


//=====================
//         OR
//=====================

int x86OR_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86OR_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86OR_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86OR_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86OR_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	*out = r5900ORI(rT0, rT0, val );				// EAX  |= 0x000000ll

	*skip += 1;
	return 2;
}


int x86OR_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		* out      = r5900LUI(rS0, sval);				// S0   = 0xuuuu0000

		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// And encode that...
		*(out + 1) = r5900ORI(rS1, rS0, sval);			// S0   = 0xuuuullll
		*(out + 2) = r5900OR(rT0, rT0, rS1);			// EAX |= 0xuuuullll
		
		*skip += 3;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));

		// Encode it...
		*(out) = r5900ORI(rT0, rT0, sval);			// EAX |= 0x0000llll

		*skip += 1;
		return 3;
	}
}


//=====================
//        AND
//=====================

int x86AND_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AND_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AND_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AND_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AND_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	* out      = r5900ADDIU(rS0, rR0, 0xFF00 | val );		// S0  = 0xFFFFFFll
	*(out + 1) = r5900AND(rT0, rT0, rS0);

	*skip += 2;
	return 2;
}


int x86AND_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate word value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		* out      = r5900LUI(rS0, sval);			// S0   = uuuu0000

		// Get lower immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));
	
		// And encode that...
		*(out + 1) = r5900ORI(rS1, rS0, sval);		// S0   = uuuullll
		*(out + 2) = r5900AND(rT0, rT0, rS1);		// EAX &= uuuullll

		*skip += 3;
		return 5;
	}
	else
	{
		// Get immediate word value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));		

		// Encode it...
		* out      = r5900LUI(rS0, 0xFFFF);					// S0   = 0xFFFF0000
		*(out + 1) = r5900ORI(rS1, rS0, sval);				// S0   = 0xFFFFllll
		*(out + 2) = r5900AND(rT0, rT0, rS1);				// EAX &= 0xFFFFllll

		*skip += 3;
		return 3;
	}
}


//=====================
//        XOR
//=====================

int x86XOR_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86XOR_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86XOR_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86XOR_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86XOR_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	uchar val;

	// Get immediate byte value.
	val = *(in + 1);

	// Encode it...
	*out = r5900XORI(rT0, rT0, val);		// EAX ^= 0x000000ll

	*skip += 1;
	return 2;
}


int x86XOR_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	ushort sval;

	if (gIsOpSize32)
	{
		// Get upper immediate short value.
		sval = MAKE_WORD(*(in + 4), *(in + 3));

		// Encode it...
		*out = r5900LUI(rS0, sval);					// S0   = 0xuuuu0000

		// Get lower immediate short value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));		
		
		// And encode that...
		*(out + 1) = r5900ORI(rS1, rS0, sval);		// S0   = 0xuuuullll
		*(out + 2) = r5900XOR(rT0, rT0, rS1);		// EAX ^= 0xuuuullll

		*skip += 3;
		return 5;
	}
	else
	{
		// Get lower short value.
		sval = MAKE_WORD(*(in + 2), *(in + 1));		
		
		// Encode it...
		*out = r5900XORI(rT0, rT0, sval);			// EAX ^= 0x0000llll

		*skip += 1;
		return 3;
	}
}


