//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "config.h"
#include "types.h"
#include "x86.h"
#include "x86misc.h"
#include "r59basic.h"


//=====================
//        CMP
//=====================

int x86CMP_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMP_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMP_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMP_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMP_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMP_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//        TEST
//=====================

int x86TEST_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86TEST_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86TEST_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86TEST_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//        XCHG
//=====================

int x86XCHG_r32r32(uchar reg1, uchar reg2, ulong *out,	ulong *skip)
{
	// Encode it...
	* out      = r5900OR(rS0, rR0, reg1);		// S0   = reg1
	*(out + 1) = r5900OR(reg1, rR0, reg2);		// reg1 = reg2
	*(out + 2) = r5900OR(reg2, rR0, rS0);		// reg2 = S0

	*skip += 3;
	return 1;
}

// This function overwrites 3 temp regs, S0 & S1 & AT
int x86XCHG_r16r16(uchar reg1, uchar reg2, ulong *out, ulong *skip)
{
	// Encode it...
											// reg1 == YYYYyyyy  reg2 == ZZZZzzzz
											//
	* out      = r5900ANDI(rS0, reg1, 0xFFFF);		// S0   = 0x0000yyyy
	*(out + 1) = r5900ANDI(rS1, reg2, 0xFFFF);		// S1   = 0x0000zzzz
	*(out + 2) = r5900XOR (rAT, reg1, rS0);			// AT   = 0xYYYY0000
	*(out + 3) = r5900OR  (reg1, rAT, rS1);			// reg1 = 0xYYYYzzzz
	*(out + 4) = r5900XOR (rAT, reg2, rS1);			// reg2 = 0xZZZZ0000
	*(out + 5) = r5900OR  (reg2, rAT, rS0);			// reg2 = 0xZZZZyyyy

	*skip += 6;
	return 1;
}

int x86XCHG_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;

/*	uchar reg;
	uint size;
	uint pos,delta;


	// Decode the REG field from the Mod R/M Byte.
	reg  = x86DecodeRegField(*(in + 1));
	
	pos = *skip;

	// Encode the reading of the byte operand from memory
	memReadByte(in + 1, out, skip, rAT);
	delta = *skip - pos;

	// Encode the XCHGing of the two bytes
												  // REG == 0xYYYYYYyy  rAT = 0x000000zz
	*(out + delta)     = r5900ANDI(rS0, reg, 0x00FF);		// S0  = 0x000000yy
	*(out + delta + 1) = r5900XOR (rS1, reg, rS0);			// S1  = 0xYYYYYY00
	*(out + delta + 2) = r5900OR  (reg, rS1, rAT);			// reg = 0xYYYYYYzz
	*(out + delta + 3) = r5900OR  (rAT, rR0, rS0);			// rAT = 0x000000yy
	*skip += 4;

	// Encode the writing of the resulting byte to memory
	delta = *skip - pos;
	size = memWriteByte(in + 1, out + delta, skip, rAT);
	
	return 1 + size; // XCHG is 1 byte
	*/
}


int x86XCHG_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;

	/*
	uchar reg;
	uint size;
	uint pos,delta;

	// Decode the REG field from the Mod R/M Byte.
	reg  = x86DecodeRegField(*(in + 1));
	
	pos = *skip;

	if (gIsOpSize32)
	{
		// Encode the reading of the dword operand from memory
		memReadDword(in, out, skip, rAT);
		delta = *skip - pos;
	
		// Call a generic version of XCHG
		x86XCHG_r32r32(rAT, reg, out + delta, skip);
		delta = *skip - pos;

		// Encode the writing of the resulting dword to memory
		size = memWriteDword(in + 1, out + delta, skip, rAT);
	
		return 1 + size; // XCHG is 1 byte;
	}
	else
	{
		// Encode the reading of the word operand from memory
		memReadWord(in + 1, out, skip, rAT);
		delta = *skip - pos;
	
		// Encode the XCHGing of the two words
											// REG == YYYYyyyy  MEM == ZZZZzzzz
		*(out + delta)     = r5900ANDI(rS0, reg, 0xFFFF);		// S0  = 0x0000yyyy
		*(out + delta + 1) = r5900XOR (rS1, reg, rS0);			// S1  = 0xYYYY0000
		*(out + delta + 2) = r5900OR  (reg, rS0, rAT);			// reg = 0xYYYYzzzz
		*(out + delta + 3) = r5900OR  (rAT, rR0, rS0);			// rAT = 0xZZZZyyyy
		*skip += 4;
		
		delta = *skip - pos;

		// Encode the writing of the resulting word to memory
		size = memWriteWord(in + 1, out + delta, skip, rAT);

		return 1 + size; // XCHG is 1 byte;
	}
	*/
}


int x86XCHG_eAXeCX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT2, out, skip);
	else
		x86XCHG_r16r16(rT0, rT2, out, skip);

	return 1;
}



int x86XCHG_eAXeDX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT3, out, skip);
	else
		x86XCHG_r16r16(rT0, rT3, out, skip);

	return 1;
}


int x86XCHG_eAXeBX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT1, out, skip);
	else
		x86XCHG_r16r16(rT0, rT1, out, skip);

	return 1;
}


int x86XCHG_eAXeSP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT7, out, skip);
	else
		x86XCHG_r16r16(rT0, rT7, out, skip);

	return 1;
}


int x86XCHG_eAXeBP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT6, out, skip);
	else
		x86XCHG_r16r16(rT0, rT6, out, skip);

	return 1;
}


int x86XCHG_eAXeSI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT4, out, skip);
	else
		x86XCHG_r16r16(rT0, rT4, out, skip);

	return 1;
}


int x86XCHG_eAXeDI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Call a generic version of XCHG

	if (gIsOpSize32)
		x86XCHG_r32r32(rT0, rT5, out, skip);
	else
		x86XCHG_r16r16(rT0, rT5, out, skip);

	return 1;
}



//=====================
//     String Ops
//=====================

int x86MOVSB_XbYb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86MOVSWD_XvYv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMPSB_XbYb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CMPSWD_XvYv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86STOSB_YbAL(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86STOSWD_YveAX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86LODSB_ALXb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86LODSWD_eAXXv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SCASB_ALYb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SCASWD_eAXYv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//  LEA/LDS/LES,etc.
//=====================

int x86LEA_GvM(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86LDS_GvMp(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86LES_GvMp(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//      REP/REPNE
//=====================

int x86REP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86REPNE(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//        NOP
//=====================

int x86NOP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Encode it...
	*out   = r5900SLL(rR0, rR0, 0);					// SLL r0, r0, 0  ->  0x00000000  ->  NOP
	*skip += 1;
	return 1;
}



//=====================
//  Segment Overrides
//=====================

int x86Seg_CS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Setup segment override...
	gIsSegOverride = TRUE;
	gSegOvrCounter = 2;
	gOverrideSeg   = xCS;

	// Check if we need to up addrSize's or opSize's counter due to this prefix...
	if (gAddrSizeCounter) gAddrSizeCounter++;
	if (gOpSizeCounter  ) gOpSizeCounter++;

	return 1;
}


int x86Seg_DS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Setup segment override...
	gIsSegOverride = TRUE;
	gSegOvrCounter = 2;
	gOverrideSeg   = xDS;

	// Check if we need to up addrSize's or opSize's counter due to this prefix...
	if (gAddrSizeCounter) gAddrSizeCounter++;
	if (gOpSizeCounter  ) gOpSizeCounter++;

	return 1;
}


int x86Seg_ES(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Setup segment override...
	gIsSegOverride = TRUE;
	gSegOvrCounter = 2;
	gOverrideSeg   = xES;

	// Check if we need to up addrSize's or opSize's counter due to this prefix...
	if (gAddrSizeCounter) gAddrSizeCounter++;
	if (gOpSizeCounter  ) gOpSizeCounter++;

	return 1;
}


int x86Seg_SS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Setup segment override...
	gIsSegOverride = TRUE;
	gSegOvrCounter = 2;
	gOverrideSeg   = xSS;

	// Check if we need to up addrSize's or opSize's counter due to this prefix...
	if (gAddrSizeCounter) gAddrSizeCounter++;
	if (gOpSizeCounter  ) gOpSizeCounter++;

	return 1;
}


int x86Seg_FS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Setup segment override...
	gIsSegOverride = TRUE;
	gSegOvrCounter = 2;
	gOverrideSeg   = xFS;

	// Check if we need to up addrSize's or opSize's counter due to this prefix...
	if (gAddrSizeCounter) gAddrSizeCounter++;
	if (gOpSizeCounter  ) gOpSizeCounter++;

	return 1;
}


int x86Seg_GS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	// Setup segment override...
	gIsSegOverride = TRUE;
	gSegOvrCounter = 2;
	gOverrideSeg   = xGS;

	// Check if we need to up addrSize's or opSize's counter due to this prefix...
	if (gAddrSizeCounter) gAddrSizeCounter++;
	if (gOpSizeCounter  ) gOpSizeCounter++;

	return 1;
}



//=====================
//  Decimal/ASCII Adj
//=====================

int x86AAA(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AAD(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AAM(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86AAS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86DAA(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86DAS(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//     Flag Control
//=====================

int x86CMC(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CLC(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86STC(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CLI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86STI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CLD(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86STD(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SAHF(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86LAHF(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}



//=====================
//       Others
//=====================

int x86BOUND_GvMa(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CBW(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86CWD(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SALC(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86XLAT(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86LOCK(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86HLT(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86WAIT(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


